package maildb

import (
	"database/sql"
	"encoding/json"
	"errors"
	"fmt"
	mysqlmgr "server/db/mysql"
	"server/msg"
)

type Mail struct {
	ID            int             `json:"id"`
	CreateTime    sql.NullTime    `json:"create_time"`
	MailType      int             `json:"mail_type"`
	UserID        sql.NullString  `json:"user_id"`
	MailInfo      json.RawMessage `json:"mail_info"`
	CheckStatus   int             `json:"check_status"`
	ReceiveStatus int             `json:"receive_status"`
}

// CreateMail 创建新邮件
// 参数:
// - mailType: 邮件类型(1=系统，2=个人)
// - userID: 用户ID(当mailType=2时必须提供)
// - mailInfo: 邮件内容(JSON格式)
// 返回:
// - 新邮件的ID
// - 错误信息
func CreateMail(mailType int, userID string, mailInfo json.RawMessage) (int64, error) {
	// 验证参数
	if mailType != 1 && mailType != 2 {
		return 0, errors.New("invalid mail_type, must be 1 or 2")
	}

	if mailType == 2 && userID == "" {
		return 0, errors.New("user_id is required when mail_type is 2")
	}

	// 准备SQL语句
	query := "INSERT INTO mail (mail_type, user_id, mail_info) VALUES (?, ?, ?)"
	if mailType == 1 {
		// 系统邮件不需要user_id
		result, err := mysqlmgr.Insert(query, mailType, nil, mailInfo)
		if err != nil {
			return 0, fmt.Errorf("failed to create system mail: %v", err)
		}
		return result, err
	} else {
		// 个人邮件需要user_id
		result, err := mysqlmgr.Insert(query, mailType, userID, mailInfo)
		if err != nil {
			return 0, fmt.Errorf("failed to create personal mail: %v", err)
		}
		return result, err
	}
}

// UpdateCheckStatus 更新邮件查看状态
// 参数:
// - mailID: 邮件ID
// - status: 新的查看状态(0=未查看，1=已查看)
// 返回:
// - 更新的行数
// - 错误信息
func UpdateCheckStatus(mailID int, status int) (int64, error) {
	if status != 0 && status != 1 {
		return 0, errors.New("invalid check_status, must be 0 or 1")
	}

	query := "UPDATE mail SET check_status = ? WHERE id = ?"
	result, err := mysqlmgr.Update(query, status, mailID)
	if err != nil {
		return 0, fmt.Errorf("failed to update check status: %v", err)
	}
	return result, err
}

// UpdateReceiveStatus 更新邮件领取状态
// 参数:
// - mailID: 邮件ID
// - status: 新的领取状态(0=未领取，1=已领取)
// 返回:
// - 更新的行数
// - 错误信息
func UpdateReceiveStatus(mailID int, status int) (int64, error) {
	if status != 0 && status != 1 {
		return 0, errors.New("invalid receive_status, must be 0 or 1")
	}

	query := "UPDATE mail SET receive_status = ? WHERE id = ?"
	result, err := mysqlmgr.Update(query, status, mailID)
	if err != nil {
		return 0, fmt.Errorf("failed to update receive status: %v", err)
	}

	return result, err
}

func GetMailsByType(mailType int, userID string) ([]*msg.MailInfo, error) {
	var rows *sql.Rows
	var err error

	// 基础查询语句
	query := "SELECT id, create_time, mail_type, user_id, mail_info, check_status, receive_status FROM mail WHERE mail_type = ?"

	// 根据mail_type决定是否添加user_id条件
	if mailType == 2 {
		if userID == "" {
			return nil, fmt.Errorf("user_id is required when mail_type is 2")
		}
		query += " AND user_id = ?"
		rows, err = mysqlmgr.Query(query, mailType, userID)
	} else {
		rows, err = mysqlmgr.Query(query, mailType)
	}

	if err != nil {
		return nil, fmt.Errorf("query failed: %v", err)
	}
	defer rows.Close()

	var mailList []*msg.MailInfo

	for rows.Next() {
		var mail Mail
		err := rows.Scan(
			&mail.ID,
			&mail.CreateTime,
			&mail.MailType,
			&mail.UserID,
			&mail.MailInfo,
			&mail.CheckStatus,
			&mail.ReceiveStatus,
		)
		if err != nil {
			return nil, fmt.Errorf("scan failed: %v", err)
		}
		mailList = append(mailList, &msg.MailInfo{
			MailJsonInfo:  string(mail.MailInfo),
			MailType:      int32(mail.MailType),
			CheckStatus:   int32(mail.CheckStatus),
			ReceiveStatus: int32(mail.ReceiveStatus),
			CreateTime:    mail.CreateTime.Time.GoString(),
		})
	}

	if err = rows.Err(); err != nil {
		return nil, fmt.Errorf("rows error: %v", err)
	}

	return mailList, nil
}

// DeleteMail 删除邮件
// 参数:
// - mailID: 要删除的邮件ID
// - userID: 执行删除操作的用户ID(用于验证个人邮件所有权)
// 返回:
// - 删除的行数
// - 错误信息
func DeleteMail(mailID int, userID string) (int64, error) {
	// 首先查询邮件信息以确定类型和所有权
	var mail Mail
	query := "SELECT mail_type, user_id FROM mail WHERE id = ?"
	err := mysqlmgr.QueryRow(query, mailID).Scan(&mail.MailType, &mail.UserID)
	if err != nil {
		if err == sql.ErrNoRows {
			return 0, errors.New("mail not found")
		}
		return 0, fmt.Errorf("failed to query mail info: %v", err)
	}

	// 检查是否为系统邮件
	if mail.MailType == 1 {
		return 0, errors.New("system mail cannot be deleted")
	}

	// 检查个人邮件所有权
	if mail.MailType == 2 {
		// 验证userID是否匹配
		if !mail.UserID.Valid || mail.UserID.String != userID {
			return 0, errors.New("you can only delete your own mail")
		}
	}

	// 执行删除操作
	deleteQuery := "DELETE FROM mail WHERE id = ?"
	if mail.MailType == 2 {
		// 个人邮件需要验证用户ID
		deleteQuery += " AND user_id = ?"
		result, err := mysqlmgr.Delete(deleteQuery, mailID, userID)
		if err != nil {
			return 0, fmt.Errorf("failed to delete mail: %v", err)
		}
		return result, err
	}

	// 理论上不会执行到这里，因为前面已经处理了所有情况
	return 0, errors.New("invalid mail type")
}
